const std = @import("std");

const mpsc = @import("../mpsc.zig");
const queue = @import("queue.zig");

pub const messages = @import("messages.zig");

pub fn MsgQueueUnmanaged(comptime T: type) type {
    return struct {
        const Self = @This();

        queue: queue.Queue(T),
        // consumer_mutex: std.Thread.Mutex = std.Thread.Mutex{},
        cond: std.Thread.Condition = std.Thread.Condition{},

        pub fn init(buffer: []mpsc.Slot(T)) Self {
            return .{
                .queue = queue.Queue(T).init(buffer),
            };
        }

        pub fn enqueue(self: *Self, value: T) void {
            self.queue.blockingEnqueue(value);
            std.log.info("enqueue: SENDING signal", .{});
            self.cond.signal();
        }

        pub fn dequeue(self: *Self) T {
            var m = std.Thread.Mutex{};
            m.lock();
            defer m.unlock();

            while (true) {
                if (self.queue.dequeue()) |val| return val;
                std.log.info("dequeue: STARTING consumer condition wait", .{});
                self.cond.wait(&m);
                std.log.info("dequeue: RECEIVED signal", .{});
            }
        }
    };
}

pub fn MsgQueueManaged(comptime T: type) type {
    return struct {
        const Self = @This();

        allocator: std.mem.Allocator,
        buffer: []mpsc.Slot(T),
        msg_queue: MsgQueueUnmanaged(T),

        pub fn init(allocator: std.mem.Allocator, capacity: usize) !Self {
            const buffer = try allocator.alloc(mpsc.Slot(T), capacity);
            errdefer allocator.free(buffer);

            return .{
                .allocator = allocator,
                .buffer = buffer,
                .msg_queue = MsgQueueUnmanaged(T).init(buffer),
            };
        }

        pub fn deinit(self: *Self) void {
            self.allocator.free(self.buffer);
        }

        pub fn unmanaged(self: *Self) *MsgQueueUnmanaged(T) {
            return &self.msg_queue;
        }

        pub fn enqueue(self: *Self, value: T) void {
            self.msg_queue.enqueue(value);
        }

        pub fn dequeue(self: *Self) T {
            return self.msg_queue.dequeue();
        }
    };
}
